<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Infomaker\Everyware\Support\Storage\Cache;
use Twig_Cache_Filesystem;
use Twig_CacheInterface;

/**
 * CacheLoader
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Twig
 * @since   Infomaker\Everyware\Twig\CacheLoader 0.1
 */
class ObjectCacheSystem extends Twig_Cache_Filesystem
{

    /**
     * Add suffix so we don't have to delete it in deploy every time
     *
     * @var string
     */
    private static $cacheKey = '';

    /**
     * Time in seconds to cache the output
     *
     * @var int
     */
    private static $cacheTime = 360;

    /**
     * Static function for initialization
     *
     * @param string $cacheKeyPrefix
     * @param int    $cacheTime
     *
     * @return static
     */
    public static function setup(string $cacheKeyPrefix, int $cacheTime = 60): ObjectCacheSystem
    {
        static::$cacheKey = $cacheKeyPrefix;
        static::$cacheTime = $cacheTime;

        return new static(static::$cacheKey);
    }

    /**
     * Writes the compiled template to cache.
     *
     * @param string $key The cache key
     * @param string $content The template representation as a PHP class
     *
     * @return void
     */
    public function write($key, $content): void
    {
        Cache::set($this->getCacheKey($key), serialize($content), static::$cacheTime);
    }

    /**
     * Create a cache key combined with the key prefix provided
     * during setup and a hashed version of the native key provided by Twig
     *
     * @param $key
     *
     * @return string
     */
    protected function getCacheKey(string $key): string
    {
        return static::$cacheKey . '_' . md5($key);
    }

    /**
     * Loads a template from the cache.
     *
     * @param string $key The cache key
     *
     * @return void
     */
    public function load($key): void
    {
        if (false !== ($content = Cache::get($this->getCacheKey($key)))) {
            eval('?>' . unserialize($content));
        }
    }
}
