<?php
namespace Infomaker\Everyware\Twig\Tests\Features;

use PHPUnit\Framework\TestCase;
use PHPUnit\Framework\MockObject\Generator;
use Behat\Behat\Context\SnippetAcceptingContext;
use Infomaker\Everyware\Twig\EveryTwig;
use Infomaker\Everyware\Twig\View;
use Infomaker\Everyware\Twig\FilesystemLoader;
use Infomaker\Everyware\Twig\ViewSetup;
use Infomaker\Everyware\Twig\Tests\src\TwigFunctions;
use Infomaker\Everyware\Support\Str;


/**
 * This is a behat test class, using Gherkin descriptions.
 * Further explanation of what is being tested and method descriptions can be found in the .feature files.
 * 
 * --!! IMPORTANT DO NOT CHANGE THE DOC BLOCKS !!--
 */
class RenderViewContext implements SnippetAcceptingContext
{
    private $everyTwig;
    private $fileLoader;
    private $viewSetup;
    private $loader;
    private $functions = [];

    public function __construct()
    {
    }

    /**
     * @Given viewsetup can be initiated
     */
    public function viewSetup_can_be_initiated()
    {
        $this->viewSetup = ViewSetup::getInstance();
    }
    /**
     * @Then a loader can be set
     */
    public function a_loader_can_be_set()
    {
        $methods = [
            'getCache',
            'getPathCacheKey',
            'setCache'
        ];
        $this->viewSetup->setLoader((new Generator())->getMock(FilesystemLoader::class, $methods));
    }

    /**
     * @Then EveryTwig can be set
     */
    public function everyTwig_can_be_set()
    {
        $this->viewSetup->setEveryTwig(EveryTwig::getInstance());
    }

    /**
     * @Then the folder path :path and :namespace can be added
     */
    public function the_folder_path_and_namespacetest_exists($path, $namespace)
    {
        $this->viewSetup->registerTwigFolder($namespace, __DIR__ . '/../' . $path);
        $this->loader = $this->fileLoader;
    }
    /**
     * @Then I can init EveryTwig
     */
    public function everyTwig_can_be_initiated()
    {
        $this->viewSetup->init();
    }
    /**
     * @Then a view(:template) can be generated
     */
    public function a_view_can_be_rendered($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertEquals('<p>Test</p>', $view);
        TestCase::assertInternalType('string', $view);
    }
    /**
     * @Then PHP functions can be added
     */
    public function wpFunctions_can_be_added_to_the_view()
    {
        $twigFunctions = new TwigFunctions();
        foreach ( \get_class_methods( $twigFunctions ) as $funcName ) {
            $this->viewSetup->addFunction( Str::snake( $funcName ), [ $twigFunctions, $funcName ] );
        }
    }

    /**
     * @Then the twig template :template can use PHP functions
     */
    public function the_template_can_use_php_functions($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('Does this work', $view);
        TestCase::assertContains('Sure hope so!', $view);
        TestCase::assertInternalType('string', $view);
    }

    /**
     * @Then Globals can be added :key :value
     */
    public function globals_can_be_added_to_twig(string $key, $value)
    {
        $this->viewSetup->addGlobal($key, $value);
    }

    /**
     * @Then the twig template :template can use Globals
     */
    public function the_twig_template_can_use_globals($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('Does', $view);
        TestCase::assertContains('This', $view);
        TestCase::assertContains('Work', $view);
        TestCase::assertInternalType('string', $view);
    }

    /**
     * @Then filters can be added :filter
     */
    public function filters_can_be_added($filter)
    {
        $callable = function($string) {
            return strtoupper($string);
        };

        $this->viewSetup->addFilter($filter, $callable);
    }

    /**
     * @Then the twig template :template can use filters
     */
    public function the_twig_template_can_use_filters($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('DOES', $view);
        TestCase::assertContains('THIS', $view);
        TestCase::assertContains('WORK', $view);
        TestCase::assertInternalType('string', $view);
    }

    /**
     * @Then the twig template :template can use filters, globals and PHP functions
     */
    public function theTwigTemplateTestallTwigCanUseFiltersGlobalsAndPhpFunctions($template)
    {
        $view = View::generate('@namespaceTest/' . $template);
        TestCase::assertNotNull($view);
        TestCase::assertContains('DOES', $view);
        TestCase::assertContains('THIS', $view);
        TestCase::assertContains('WORK', $view);
        TestCase::assertContains('Uno', $view);
        TestCase::assertContains('Dos', $view);
        TestCase::assertContains('Tres', $view);
        TestCase::assertContains('Does this work', $view);
        TestCase::assertContains('Sure hope so!', $view);
        TestCase::assertInternalType('string', $view);
    }
}