<?php
use PHPUnit\Framework\TestCase;
use Infomaker\Everyware\Twig\EveryTwig;
use Infomaker\Everyware\Twig\FilesystemLoader;
use Twig\Environment;
use function Patchwork\{redefine};
use Infomaker\Everyware\Support\Storage\Cache;
/**
 * EveryTwigTest
 * @group EveryTwig
 */
class EveryTwigTest extends TestCase
{
    private $everyTwig;

    public function setUp()
    {
        parent::setUp();

        $this->everyTwig = EveryTwig::getInstance();
        $this->prophet = new \Prophecy\Prophet;
    }
    protected function tearDown()
    {
        $this->prophet->checkPredictions();
    }

    /** @test */
    public function a_new_instance_can_be_created()
    {
        $this->assertInstanceOf(EveryTwig::class, EveryTwig::getInstance());
    }

    /** @test */
    public function it_can_be_initialized()
    {
        $loader = new FilesystemLoader();
        $twig = $this->everyTwig->init($loader);
        $this->assertInstanceOf(Environment::class, $twig);
    }
    
    /**
     * @test
     */
    public function a_view_can_be_rendered()
    {
        // Create a stub for the SomeClass class.
        $file = $this->getMockBuilder(FilesystemLoader::class)
            ->setMethods(['getCache', 'setCache', 'getPathCacheKey'])
            ->getMock();

        // Configure the stub.
        $file->expects($this->once())
            ->method('getCache');
        $file->expects($this->once())
            ->method('getPathCacheKey');
        $file->expects($this->once())
            ->method('setCache');

        $file->addPathAndSubfolders(__DIR__ . '/../src', 'test');
        $loader = $file;
        $this->everyTwig->init($loader);
        $view = $this->everyTwig->view('@test/testTemplate.twig');
        $this->assertNotNull($view);
        $this->assertEquals('<p>Test</p>', $view);
        $this->assertInternalType('string', $view);

    }
}
